// main.js contains most of the javascript functionality required for info pages to run in the webview

var modules = [
	// 3rd party libraries
	'lib/handlebars.runtime.min.js', // template system http://handlebarsjs.com/
	'lib/jquery.fullpage.js', // full page scrolling https://alvarotrigo.com/fullPage/
	'lib/jquery.cycle2.min.js', // cycling slideshow plugin http://jquery.malsup.com/cycle2/
	'lib/FastClick.js', // Polyfill to remove click delays on browsers with touch UIs https://github.com/ftlabs/fastclick
	'lib/jquery.collapse.js',

	// these are pulled from gdoc https://docs.google.com/spreadsheets/d/1WMIf2VwaywFFT9lUK_njvVlLflsdeXeVvd5I1h4SZE8/edit#gid=396740001
	'data/variables.js',

	// this is current info branch, version, datestamp
	'version.js'
]

// pull in the data matching the block passed in from the containing page (eg. parents.html, walkthrough.html)
if (blockName) {
	modules.push('data/' + blockName + '.js');
	if (blockName === "parents") {
		// parents has and extra reviews sheet; pull that in
		modules.push('data/reviews.js');
	}
}

require (['jquery.js'], function(jquery) {
	require(modules,
		function   (Handlebars, fullPage, cycle2)  {

		console.log("Info Version " + version + " " + date)
		console.log("Loading block: " + blockName);

		this.Handlebars = Handlebars;
		this.strings = data;
		this.variables = variables;
		if (typeof(reviews) != "undefined") {
			this.reviews = reviews; 
		}

		$(document).ready(function () {
			$(function() {
				FastClick.attach(document.body);
			});
			// apply display language dependent CSS 
			$('head').append('<link rel="stylesheet" type="text/css" href="css/lang/' + _lang + '.css"/>');
			// if dialog page, get dialog styles 
			if (blockName === "dialogs") {
				$('head').append('<link rel="stylesheet" type="text/css" href="css/dialog.css"/>');
			}
			// apply product dependent styles
			$('head').append('<link rel="stylesheet" type="text/css" href="css/product/' + _product + '.css"/>');
		});
		
		require(['src/templates.js'],
			function (Templates) {
				var strings = populateTemplate(blockName, this.strings, this.variables, this.reviews);
				strings.version = version;

				makeButtonActions();
				if (_debug  ) {
					showVersionText();
				}

				atLastSection = false;

				// define oneLongSection if not done so
				try { if (oneLongSection) null;
				} catch(e) { oneLongSection = false; }

				// define hideNavigation if not done so
				try { if (hideNavigation) null;
				} catch(e) { hideNavigation=false; }

				$(document).ready(function() {
					$(".collapse").collapse({
						open: function() {
				        	this.slideDown(150);
				        },
				        close: function() {
				        	this.slideUp(150);
				        }
					});
					$('#review-slideshow').cycle();
					$('#fullpage').fullpage({
						bigSectionsDestination: "top",
						navigation: !hideNavigation,
						navigationPosition: "right",
						controlArrows: false,
						// navigationTooltips: ['firstSlide', 'secondSlide'],
						verticalCentered: false,
						recordHistory: false,
						loopHorizontal: false,
						animateAnchor: false,

						fitToSection:false, // otherwise long sections (like FAQ) auto scroll to top after a short delay)

						autoScrolling: !oneLongSection,

						scrollingSpeed: 600,
						lazyLoading: false,
						afterLoad: function(anchorLink, index){
							_anchorLink = anchorLink;
							// console.log("arrived at " + _anchorLink);

							// if not a dialog page, then report this page action to the containing app 
							if (_panel == undefined) {
								var params = "action=arrive_at_section&anchor="+_anchorLink+"&x=true";
								var xobj = new XMLHttpRequest();
        						xobj.open('GET', "http://localhost:8087?" + params, true);
        						xobj.send(null);
        					}

				        },
				        onLeave: function(index, nextIndex, direction){

				    		atLastSection = (nextIndex == strings.sectionCount);

				    		if (atLastSection == true) {
				    			$("#next-button").hide()
				    		} else {
				    			$("#next-button").show()
				    		}

				    		// clone the next-button to restart its animation
							var el     = $("#next-button"),  
							    newone = el.clone(true);
							el.before(newone);
							el.remove();
							setTimeout(function() {
								$.fn.fullpage.moveSlideLeft();
							}, 1);
				        }

					});

					// $('video').mediaelementplayer();
				});
		});
	});
});


// get parameter out of URL 
function getURLParameter(name) {
    return decodeURIComponent((new RegExp('[?|&]' + name + '=' + '([^&;]+?)(&|#|;|$)')
    	.exec(location.href)||[,""])[1].replace(/\+/g, '%20')) || null;
}	
var _lang = getURLParameter("lang")  // en|es|zh-Hans ....
var _product = getURLParameter("product") // FE|FS|FC|FEC|FECH....
var _user = getURLParameter("user") // user|paid
var _platform = getURLParameter("platform") // apple|android
var _locale = getURLParameter("locale") // region codes, or TESTING
var _debug = getURLParameter("debug") // whether or not to show debug info

var _panel = getURLParameter("panel") // single panel mode rather than a full page

if (_lang == null ) {
	_lang = "en"
}
if (_product == null ) {
	_product = "FE"
}
if (_user == null ) {
	_user = "free"
}
if (_platform != "apple" && _platform != "android") {
	_platform = "apple"
}

_debug = _debug === "true";

function extractLanguage(block, strings, lang) {
	var base = strings["sections"];
	if (typeof block === 'string') {
		base = strings[block]["sections"];
	}
	var sections = {};
	for (var section in base) {
		sections[section] = {
			anchor: base[section].anchor,
		};
		for (var key in base[section].items) {
			var text = base[section].items[key][lang];
			// if it matches [en] or [any language code] then get it again, using that code
			if (text) {
				var match = text.match("\[[a-zA-Z-]+\]")
				if (match) {
					var _lang = text.slice(1,-1)
					text = base[section].items[key][_lang]
				}
			}
			if (text === undefined) {
				// if no translation found
				// default to english
				text = base[section].items[key]['en']
				// and warn
				console.warn("No " + lang + " translation found for " 
								+ section + "." + key);
			}
			sections[section][key] = text;

			// replace carriage returns with <br>s
			if (sections[section][key] != undefined) {
				sections[section][key] = 
				sections[section][key].replace(/\n/g, "<br/>");
			}
		}
	}
	return {sections: sections};
}

function shouldInclude(filter, product) {
	var p = "|" + product + "|";

	var result = false;
	if (filter=="all") { result= true; }
	if (filter.lastIndexOf('+', 0) === 0) {
		result = filter.indexOf(p) !== -1;
	} else if (filter.lastIndexOf('-', 0) === 0) {
		result = filter.indexOf(p) === -1;
	}

	// console.log(filter, product, result);
	return result;
}


function filterByProduct(block, strings, product) {
	var base = strings;
	if (typeof block === 'string') {
		base = strings[block];
	}
	console.log("block = " + block)
	// console.log(block, strings, product);
	console.log(strings);
	var sections = {};
	for (var section in base) {
		// console.log(base[section]);
		// console.log(base[section].config);
		// check that this section contains a config
		if (base[section].config === undefined) {
			// whoops no config! 
			console.log(base[section]);
			console.log("No config found for above section. Make sure there is a row with an 'anchor' column matching this row's 'section' column.")
		}
		if (shouldInclude(base[section].config.filter, product)) {
			var anchor = base[section].config.anchor;
			delete base[section].config;
			sections[section] = {
				anchor: anchor,
				items: {}
			};
			for (var filter in base[section].items) {

				if (shouldInclude(filter, product)) {
					for (var item in base[section].items[filter]) {
						sections[section].items[item] = base[section].items[filter][item];
					}
				}
				delete base[section].items[filter];
			}
			delete base[section];
			// console.log("show section", section, sections[section].anchor);

		} else {
			delete base[section];
			// console.log("hide section", section);
		}

	}
	base["sections"] = sections;
	return strings;
}

function extractVariables(variables, product) {
	var out = {};
	for (var key in variables) {
		if (variables[key].hasOwnProperty(product)) {
			out[key] = variables[key][product];
		}
	}
	return out;
}

function insertVariables(strings, variables) {
	for (var section in strings.sections) {
		for (var item in strings.sections[section]) {
			for (var variable in variables) {
				var s = strings.sections[section][item];
				if (s) {
					s = s.replace(variable, variables[variable]);
				}
				strings.sections[section][item] = s;
			}
		}
	}
	return strings;
}

function finalizeReviews(reviews) {

	delete reviews.sections.reviews.anchor;
	for (var key in reviews.sections.reviews) {
		if (reviews.sections.reviews[key] === undefined) {
			delete reviews.sections.reviews[key];
		}
	}

	var out = [];
	for (var key in reviews.sections.reviews) {
		out.push(reviews.sections.reviews[key]);
	}
	return out;
}

function populateTemplate(block, strings, variables, reviews) {

	// get variables for chosen product
	var variables = extractVariables(variables, _product);

	// filter block according to current product
	strings= filterByProduct(block, strings, _product);
	// console.log( "FILTERED BY PRODUCT");
	// console.log(strings);

	// get strings in the chosen language 
	strings = extractLanguage(block, strings, _lang);
	// console.log( "EXTRACT LANG");
	// console.log(strings);

	// insert variables into strings
	strings = insertVariables(strings, variables);
	// console.log( "INSERT VARIABLES");
	// console.log(strings);

	// extract page details from strings
	// strings.sectionAnchors = getSectionAnchors(strings);

	// add table of contents	
	if (strings.sections.reviews != undefined) {
		// get reviews in chosen language
		// HACK fragile - what if no sections.reviews defined
		if (reviews != undefined) {
			reviews = filterByProduct("review", reviews, _product);
			reviews = extractLanguage("review", reviews, _lang);
			reviews = finalizeReviews(reviews);
			strings.sections.reviews.reviews = reviews;
		}
	}

	this.Handlebars.registerHelper('tableOfContentsButtonName', function(pageAnchor) {
  		return strings.sections.contents[pageAnchor];
	});

	function productMatch(list) {
		var products = list.split(",");
		var contains = $.inArray(_product, products)
		return contains != -1
	}

	function includes(productList, options) {
		var contains = productMatch(productList);
		return contains ? options.fn(this):options.inverse(this);
	};

	function excludes(productList, options) {
		var contains = productMatch(productList);
		return contains ? options.inverse(this):options.fn(this);
	};

	this.Handlebars.registerHelper('include', includes);
	this.Handlebars.registerHelper('exclude', excludes);

	function freeUser(options) {
		var isFreeUser = _user == "free"
		return isFreeUser ? options.fn(this):options.inverse(this);
	};
	this.Handlebars.registerHelper('freeUser', freeUser);

	function apple(options) {
		var isApple = _platform == "apple"
		return isApple ? options.fn(this):options.inverse(this);
	};
	this.Handlebars.registerHelper('apple', apple);

	function android(options) {
		var isAndroid = _platform == "android"
		return isAndroid ? options.fn(this):options.inverse(this);
	};
	this.Handlebars.registerHelper('android', android);

	function china(options) {
		var isChina = _locale == "CN"
		return isChina ? options.fn(this):options.inverse(this);
	};
	this.Handlebars.registerHelper('china', china);	

	var sections = {};
	// for each section, add it to the html
	for (var section in strings.sections) {
		// contents page is added later
		if (section != "contents") {
			if ( typeof Templates[section] === 'function'){
				var context = strings.sections[section];
				var html = Templates[section](context);
				sections[section] = html;
			} else {
				console.log("A section named '" + section + "' was found in the sheet but not defined in a template.")
			}
		}
	}

	// populate the template with the strings
	var html;
	if (_panel == undefined) { 
    	html = Templates[block](sections);
	} else {
    	html = sections[_panel];
	}

	
    // insert populated template into the fullpage div 
	$("div#fullpage").html(html);

	// if there's a contents page, then populate it 
	if (strings.sections.contents != undefined) {
		// count sections and index them to make contents page
		var sectionAnchors = [];
		$('.section').each(function(){
			var anchor = $(this).attr("data-anchor");
			// only include it if there is a matching row
			// within the contents section in the gdoc
			if (strings.sections.contents[anchor] != undefined) {
				sectionAnchors.push(anchor);
			}
		});
		strings.sections.contents.tableOfContentsButtons = sectionAnchors;
		var context = strings.sections.contents;
		var html = Templates.contents(context)
		$("div#fullpage").prepend(html);
	}
	
	strings.sectionCount = $('.section').length;

	console.log(strings);

	return strings
}

function makeButtonActions() {

	// whenever any button is pressed, send an <action> event to the 
	// containing Corona SDK project, with the "rel" attribute of the button
	// as the id of the action (e.g. "close", "learn_more", "send_mail", "rate_app")
    function send(params) {
        var xobj = new XMLHttpRequest();
        xobj.open('GET', "http://localhost:8087?" + params, true);
        xobj.send(null);
        return true
    }

    function close() {
        send("action=close&anchor="+_anchorLink+"&x=true")
        // scroll page to top after closing
        setTimeout(function() {
			var firstSection = $("#fullpage .section:first").attr('data-anchor');
			$.fn.fullpage.moveTo(firstSection);        	
		}, 500);
    }

	$("button").click(function() {
		var action = $(this).attr('rel');
		if (action != undefined) {
        	send("action=" + action +"&anchor="+_anchorLink+"&x=true")
        }
	});
	$("a").click(function() {
		var action = $(this).attr('rel');
		if (action != undefined) {
	        send("action=" + action +"&anchor="+_anchorLink+"&x=true")
	    }
	});
	$("input#close-button").click(function() {
		close()
	});
	$("input#next-button").click(function() {
		if (atLastSection) {
			// if at last page, then close
			close()
		} else {
			// else move down one section
			send("action=next&anchor="+_anchorLink+"&x=true")
			$.fn.fullpage.moveSectionDown();
		}
	});	

}

function showVersionText() {
	var w = window.innerWidth;
	var h = window.innerHeight;
	var ow = window.outerWidth; //including toolbars and status bar etc.
	var oh = window.outerHeight;
	console.log(w + "x" + h,ow + "x" + oh);
	$("#version_text").html(w + "x" + h + "<br/>" +version + " " + date); 
	// $("#version_text").html(version + " " + date); 
}
