// do the remote call to get the student data from schools-api
// then pass it to each view populator

requirejs.config({
  packages: [{
    name: 'moment',
    // This location is relative to baseUrl. Choose bower_components
    // or node_modules, depending on how moment was installed.
    location: '../node_modules/moment',
    main: 'moment'
  }]
});

var modules = [
    'moment',
    'src/reports/populators/dashboard.js',
    'src/reports/populators/activity.js',
    'src/reports/populators/skills.js',
    'src/reports/populators/class.js',
    'src/reports/populators/assignment.js',
    'src/reports/populators/vocab.js',
    'src/reports/populators/curriculum.js',
    // 'src/reports/unit4Module3.js'
    // 'src/reports/activityHistory.js'
]

require(modules, function (moment) {

    // console.log("moment", moment().format('YYYY'))
    // console.log("accuracyData", accuracyData)

    let dateFormat = "YYYY-MM-DD";

    function calculateDailyPlaytime(activityHistory) {
        let activities = activityHistory.data.activities;

        let first = activities.reduce((first, activity) => {
            first = Math.min(first, activity.date);
            return first
        }, 9999999999999)
        
        first = moment(first).startOf('day')
        let now = moment().startOf('day') //.subtract(40, 'days')

        // make an array of days from first until now
        let days = [];
        let current = first;
        while (current <= now) {
            days.push(current.clone())
            current.add(1, 'day')
        }
        console.log(days, "days")


        let playDays = activities.reduce((playDays, activity) => {
            let day = moment(activity.date).format(dateFormat);
            playDays[day] = playDays[day] || []
            playDays[day].push(activity.duration);
            return playDays
        }, {})

        // console.log("playDays", playDays)

        // work out total duration per playDay
        let playDurations = Object.keys(playDays).reduce((playDurations, key) => {
            playDurations[key] = playDays[key].reduce((a, b) => a + b, 0);
            return playDurations
        }, {})


        // create an array of durations for each day 
        let durations = days.map(day => {
            return playDurations[day.format(dateFormat)] || 0
        })

        console.log("durations", durations)

        return {
            days: days,
            durations: durations
        }
    }

    function populate(
        studentData, 
        classData, 
        activityHistoryData, 
        // assignmentsData,
        // curriculumData
        ) { 
        Chart.defaults.global.defaultFontSize = 19;
        Chart.defaults.global.defaultFontFamily = 'Convergence';
        Chart.defaults.global.elements.point.radius = 5;

        // let curriculumTree = reduceCurriculum(curriculumResponse)

        let accuracyData = calculateAccuracy(activityHistoryData);
        // let scoreData = calculateLessonScores(activityHistoryData);

        // move to a studentData mangler class
        // studentData.score = studentData.score.toFixed(2);

        console.log("studentData", studentData)
        studentData.data.human_duration = moment.duration(studentData.data.total_duration, "seconds").humanize();


        let dashboardData = Object.assign({}, accuracyData, studentData)

        console.log("dashboardData", dashboardData)
        populateDashboard(dashboardData)

        let skillsData = calculateLessonScores(activityHistoryData);
        populateSkillsChart(skillsData)

        let dayData = calculateDailyPlaytime(activityHistoryData);
        populateActivityChart(dayData)

        // populateActivity2Chart(Data)
        let classRankData = calculateClassRank(classData)
        populateClassChart(classRankData)

        // getAssignmentResults(assignmentsData);

    }


    // do the remote calls 
    var axios = require('lib/axios.min.js')

    let studentURL = `https://api.study.cat/1/student/${_code}/${_pin}`;
    let classURL = `https://api.study.cat/1/class/${_code}`;
    let activityHistoryURL = `https://api.study.cat/1/student/${_code}/${_pin}/activity-history`;
    // let assignmentsURL = `https://api.study.cat/1/class/${_code}/activity-history/4/3`;
    // let curriculumURL = 'https://api.study.cat/test/curriculum/fe';

    axios.all([
            axios.get(studentURL, { params: {class_code: _code, pin: _pin}}),
            axios.get(classURL),
            axios.get(activityHistoryURL),
            // axios.get(assignmentsURL),
            // axios.get(curriculumURL)
        ])
    .then(axios.spread(function (
                        studentResponse, 
                        classResponse, 
                        activityHistoryResponse, 
                        // assignmentsResponse,
                        // curriculumResponse
                        ) 
        {  
        console.log('studentResponse', studentResponse.data);
        console.log('classResponse', classResponse.data);
        console.log('activityHistoryResponse', activityHistoryResponse.data);
        // console.log('assignmentsResponse', assignmentsResponse.data);

        populate(
            studentResponse.data, 
            classResponse.data, 
            activityHistoryResponse.data,
            // assignmentsResponse.data,
            // curriculumResponse.data)
        ) 

    }))
    .catch(function (error) {
        console.log(error);
    });

});