function calculateAccuracy(activityHistory) {

    let activities = activityHistory.data.activities;

    // replace with a weighted function favouring recency
    function dateWeight(date) {
        return 1
    }

    // grab just the bits we need from the activity data
    let plays = activities.map(activity => {
        let v = activity.vocab;
        let words = Object.keys(v).map(key => {
            return {
                date: activity.date,
                word: key,
                q: v[key].q,
                a: v[key].a
            }
        })
        return words
    })

    // console.log(plays);

    // a line represents an occurence of a word within a game play
    let lines = plays.reduce((lines, play) => {
        play.forEach((item) => {
            lines.push(item);
        })
        return lines
    }, [])

    // console.log("lines", lines);

    // turn into an object, indexed by word name 
    // and add up the q/a for each as we go 
    let allWords = lines.reduce((allWords, line) => {
        allWords[line.word] = allWords[line.word] || {q:0, a:0};
        allWords[line.word].q += line.q;
        allWords[line.word].a += line.a;
        return allWords
    }, {})
 
    // console.log("allWords", allWords);

    // only include the words which have one or more questions
    let minQuestions = 5;
    let words = Object.keys(allWords).reduce((words, key) => {
        if (allWords[key].q >= minQuestions) {
            words[key] = allWords[key];
        }
        return words
    }, {})

    console.log("words", words);

    // figure out total accuracy
    let accuracy = Object.keys(words).reduce((accuracy, key) => {
        accuracy.total = accuracy.total + words[key].a / words[key].q;
        accuracy.count = accuracy.count + 1;
        return accuracy
    }, {count:0, total:0} )

    console.log("accuracy", accuracy.total/accuracy.count)

    // figure out per-word accuracy

    // maintains an object on output
    // let wa = Object.keys(words).reduce((wa, key) => {
    //     wa[key] = words[key].a / words[key].q;
    //     return wa
    // }, {})

    let wa = Object.keys(words).map(key => {
        return {
            word: key,
            accuracy: words[key].a / words[key].q
        }
    }).sort((x,y) => x.accuracy - y.accuracy)

    console.log("word accuracy", wa)
 
    let bottomThreeWords = wa.slice(0, 3)
    let topThreeWords = wa.slice(-3)

    // console.log("top three", topThreeWords)
    // console.log("bottom three", bottomThreeWords)

    return {
        accuracy: (accuracy.total/accuracy.count).toFixed(2)*100,
        topWords: topThreeWords,
        bottomWords: bottomThreeWords
    }
}

function calculateLessonScores(activityHistory) {
    let activities = activityHistory.data.activities;

    // group scores by lesson
    let lessons = activities.reduce((lessons, activity) => {
        lessons[activity.lesson] = lessons[activity.lesson] || [];
        lessons[activity.lesson].push(activity.score);
        return lessons
    }, {})

    console.log("lessons", lessons);

    // figure out score averages per lesson
    let result = Object.keys(lessons).reduce((result, key) => {

        let count = lessons[key].length;
        let sum =  lessons[key].reduce((a, b) => a + b, 0);

        result.lessons.push(key)
        result.scores.push(sum/count);

        return result

    }, {lessons:[], scores:[]})
  
    // let averages = Object.keys(lessons).map(key => {
    //     let count = lessons[key].length;
    //     let sum =  lessons[key].reduce((a, b) => a + b, 0);
    //     return sum/count
    // })


    console.log("result", result);

    return result
}

function calculateDailyPlaytime(activityHistory) {
    let activities = activityHistory.data.activities;

    let days = activities.reduce((days, activity) => {
        
        let day = moment(activity.data).format("MMM Do YYYY");

        // let day = new Date(activity.date);
        console.log(day);

        days[day] = days[day] || []
        days[day].push(activity);
        return days
    }, {})

    console.log("days", days)

}